#include "collisions.h"
   
void Collisions::
resolve_all_collisions(std::vector<DeformObj> &object)
{
   unsigned int i, j, k, r;
   for(int pass=0; pass<5; ++pass){
      bool found_collisions=false;
      // check every node of every object against everything
      for(i=0; i<object.size(); ++i){
         for(j=0; j<object[i].surface_nodes.size(); ++j){
	    r=object[i].surface_nodes[j];
            found_collisions|=resolve_against_box(object[i].oldx[r], object[i].x[r], object[i].v[r]);
            for(k=0; k<object.size(); ++k){
               if(i!=k)
                  found_collisions|=resolve_against_object(object[i].oldx[r], object[i].x[r], object[i].v[r], object[k]);
	       else
                  found_collisions|=resolve_against_object(object[i].oldx[r], object[i].x[r], object[i].v[r], object[k], r);
            }
         }
      }
      if(!found_collisions)
         break;
   }
}

bool Collisions::
resolve_against_box(const Vec2f &oldx, Vec2f &x, Vec2f &v)
{
   float friction;
   bool collided=false;
   // since this geometry is simple enough, we actually will ignore oldx, and instead
   // project to the closest edge of the box
   if(x[0]<boxleft){
      friction=boxfriction*(boxleft-x[0]);
      x[0]=boxleft;
      v[0]=0;
      if(boxfriction>fabs(v[1]))
         v[1]=0;
      else
         v[1]-=copysignf(boxfriction, v[1]);
      collided=true;
   }else if(x[0]>boxright){
      friction=boxfriction*(x[0]-boxright);
      x[0]=boxright;
      v[0]=0;
      if(boxfriction>fabs(v[1]))
         v[1]=0;
      else
         v[1]-=copysignf(boxfriction, v[1]);
      collided=true;
   }
   if(x[1]<boxfloor){
      friction=boxfriction*(boxfloor-x[1]);
      x[1]=boxfloor;
      v[1]=0;
      if(boxfriction>fabs(v[0]))
         v[0]=0;
      else
         v[0]-=copysignf(boxfriction, v[0]);
      collided=true;
   }else if(x[1]>boxceiling){
      friction=boxfriction*(x[1]-boxceiling);
      x[1]=boxright;
      v[1]=0;
      if(boxfriction>fabs(v[0]))
         v[0]=0;
      else
         v[0]-=copysignf(boxfriction, v[0]);
      collided=true;
   }
   return collided;
}

bool Collisions::
resolve_against_object(const Vec2f &oldx, Vec2f &x, Vec2f &v, DeformObj &obj, int exclude_node)
{
   // for this assignment, you can ignore object-object collisions
   return false;
}

