#ifndef VEC2F_H
#define VEC2F_H

#include <cassert>
#include <cmath>
#include <iostream>

struct Vec2f
{
   float v[2];

   Vec2f(void)
   {}

   Vec2f(float value_for_all)
   { v[0]=v[1]=value_for_all; }

   Vec2f(const Vec2f &source)
   { v[0]=source.v[0]; v[1]=source.v[1]; }

   Vec2f(const float source[2])
   { v[0]=source[0]; v[1]=source[1]; }

   Vec2f(float v0, float v1)
   { v[0]=v0; v[1]=v1; }

   float &operator[](int index)
   {
      assert(0<=index && (unsigned int)index<2);
      return v[index];
   }

   const float &operator[](int index) const
   {
      assert(0<=index && (unsigned int)index<2);
      return v[index];
   }

   Vec2f operator+=(const Vec2f &w)
   { v[0]+=w.v[0]; v[1]+=w.v[1]; return *this; }

   Vec2f operator-=(const Vec2f &w)
   { v[0]-=w.v[0]; v[1]-=w.v[1]; return *this; }

   Vec2f operator*=(float scalar)
   { v[0]*=scalar; v[1]*=scalar; return *this; }

   Vec2f operator/=(float scalar)
   { v[0]/=scalar; v[1]/=scalar; return *this; }
};

inline float mag2(const Vec2f &a)
{ return a.v[0]*a.v[0] + a.v[1]*a.v[1]; }

inline float mag(const Vec2f &a)
{ return std::sqrt(mag2(a)); }

inline bool operator==(const Vec2f &a, const Vec2f &b)
{ return a.v[0]==b.v[0] && a.v[1]==b.v[1]; }

inline bool operator!=(const Vec2f &a, const Vec2f &b)
{ return a.v[0]!=b.v[0] || a.v[1]!=b.v[1]; }

inline Vec2f operator-(const Vec2f &a)
{ return Vec2f(-a.v[0], -a.v[1]); }

inline Vec2f operator+(const Vec2f &a, const Vec2f &b)
{ return Vec2f(a.v[0]+b.v[0], a.v[1]+b.v[1]); }

inline Vec2f operator-(const Vec2f &a, const Vec2f &b)
{ return Vec2f(a.v[0]-b.v[0], a.v[1]-b.v[1]); }

inline Vec2f operator*(const Vec2f &a, float scalar)
{ return Vec2f(scalar*a.v[0], scalar*a.v[1]); }

inline Vec2f operator*(float scalar, const Vec2f &a)
{ return Vec2f(scalar*a.v[0], scalar*a.v[1]); }

inline Vec2f operator/(const Vec2f &a, float scalar)
{ return Vec2f(a.v[0]/scalar, a.v[1]/scalar); }

inline float dot(const Vec2f &a, const Vec2f &b)
{ return a.v[0]*b.v[0] + a.v[1]*b.v[1]; }

inline float cross(const Vec2f &a, const Vec2f &b)
{ return a.v[0]*b.v[1]-a.v[1]*b.v[0]; }

inline Vec2f perp(const Vec2f &a)
{ return Vec2f(-a.v[1], a.v[0]); }

inline Vec2f rotate(float cosine, float sine, const Vec2f &a)
{ return Vec2f(cosine*a.v[0]-sine*a.v[1], sine*a.v[0]+cosine*a.v[1]); }

inline void normalize(Vec2f &a)
{ a/=mag(a); }

inline std::ostream &operator<<(std::ostream &out, const Vec2f &a)
{ return out<<a.v[0]<<' '<<a.v[1]; }

inline std::istream &operator>>(std::istream &in, Vec2f &a)
{ return in>>a.v[0]>>a.v[1]; }

#endif
