#ifndef VEC3I_H
#define VEC3I_H

#include <cassert>
#include <cmath>
#include <iostream>

struct Vec3i
{
   int v[3];

   Vec3i(void)
   {}

   Vec3i(int value_for_all)
   { v[0]=v[1]=v[2]=value_for_all; }

   Vec3i(const Vec3i &source)
   { v[0]=source.v[0]; v[1]=source.v[1]; v[2]=source.v[2]; }

   Vec3i(const int source[3])
   { v[0]=(int)source[0]; v[1]=(int)source[1]; v[2]=(int)source[2]; }

   Vec3i(int v0, int v1, int v2)
   { v[0]=v0; v[1]=v1; v[2]=v2; }

   int &operator[](int index)
   {
      assert(0<=index && (unsigned int)index<3);
      return v[index];
   }

   const int &operator[](int index) const
   {
      assert(0<=index && (unsigned int)index<3);
      return v[index];
   }

   Vec3i operator+=(const Vec3i &w)
   { v[0]+=w.v[0]; v[1]+=w.v[1]; v[2]+=w.v[2]; return *this; }

   Vec3i operator-=(const Vec3i &w)
   { v[0]-=w.v[0]; v[1]-=w.v[1]; v[2]-=w.v[2]; return *this; }

   Vec3i operator*=(int scalar)
   { v[0]*=scalar; v[1]*=scalar; v[2]*=scalar; return *this; }

   Vec3i operator/=(int scalar)
   { v[0]/=scalar; v[1]/=scalar; v[2]/=scalar; return *this; }
};

inline bool operator==(const Vec3i &a, const Vec3i &b)
{ return a.v[0]==b.v[0] && a.v[1]==b.v[1] && a.v[2]==b.v[2]; }

inline bool operator!=(const Vec3i &a, const Vec3i &b)
{ return a.v[0]!=b.v[0] || a.v[1]!=b.v[1] || a.v[2]!=b.v[2]; }

inline Vec3i operator-(const Vec3i &a)
{ return Vec3i(-a.v[0], -a.v[1], -a.v[2]); }

inline Vec3i operator+(const Vec3i &a, const Vec3i &b)
{ return Vec3i(a.v[0]+b.v[0], a.v[1]+b.v[1], a.v[2]+b.v[2]); }

inline Vec3i operator-(const Vec3i &a, const Vec3i &b)
{ return Vec3i(a.v[0]-b.v[0], a.v[1]-b.v[1], a.v[2]-b.v[2]); }

inline Vec3i operator*(const Vec3i &a, int scalar)
{ return Vec3i(scalar*a.v[0], scalar*a.v[1], scalar*a.v[2]); }

inline Vec3i operator*(int scalar, const Vec3i &a)
{ return Vec3i(scalar*a.v[0], scalar*a.v[1], scalar*a.v[2]); }

inline Vec3i operator/(const Vec3i &a, int scalar)
{ return Vec3i(a.v[0]/scalar, a.v[1]/scalar, a.v[2]/scalar); }

inline std::ostream &operator<<(std::ostream &out, const Vec3i &a)
{ return out<<a.v[0]<<' '<<a.v[1]<<' '<<a.v[2]; }

inline std::istream &operator>>(std::istream &in, Vec3i &a)
{ return in>>a.v[0]>>a.v[1]>>a.v[2]; }

#endif

